#! /bin/tcsh
#
# Copyright (C) 2003-2010 Intel Corporation. All rights reserved.
#
# The information and source code contained herein is the exclusive property
# of Intel Corporation and may not be disclosed, examined, or reproduced in
# whole or in part without explicit written authorization from the Company.
#

setenv MKLROOT <INSTALLDIR>/mkl
setenv CPRO_PATH {$MKLROOT:s!/mkl!!}

setenv SCRIPT_NAME ${0}

setenv MOD_NAME mod

setenv MKL_TARGET_ARCH
setenv MKL_LP64_ILP64
setenv MKL_MOD
setenv MKL_VERBOSE

if ( "${1}" == "" ) then
    goto Help
endif

while ( "$1" != "" )
   if      ( "$1" == "ia32"      ) then
       setenv MKL_TARGET_ARCH ia32
   else if ( "$1" == "intel64"   ) then
       setenv MKL_TARGET_ARCH intel64
   else if ( "$1" == "lp64"      ) then
       setenv MKL_LP64_ILP64 lp64
   else if ( "$1" == "ilp64"     ) then
       setenv MKL_LP64_ILP64 ilp64
   else if ( "$1" == ${MOD_NAME} ) then
       setenv MKL_MOD ${MOD_NAME}
   else if ( "$1" == "verbose"   ) then
       setenv MKL_VERBOSE verbose
   else
       echo
       echo "ERROR: Unknown switch '$1'"
       goto Help
   endif
   shift
end

if ( "${MKL_TARGET_ARCH}" == "" ) then
   echo
   echo "ERROR: architecture is not defined. Accepted values: ia32, intel64"
   goto Help
endif


if ( ${?LD_LIBRARY_PATH} ) then
    setenv LD_LIBRARY_PATH "${CPRO_PATH}/compiler/lib/${MKL_TARGET_ARCH}:${MKLROOT}/lib/${MKL_TARGET_ARCH}:${LD_LIBRARY_PATH}"
else
    setenv LD_LIBRARY_PATH "${CPRO_PATH}/compiler/lib/${MKL_TARGET_ARCH}:${MKLROOT}/lib/${MKL_TARGET_ARCH}"
endif

if ( ${?LIBRARY_PATH} ) then
    setenv LIBRARY_PATH "${CPRO_PATH}/compiler/lib/${MKL_TARGET_ARCH}:${MKLROOT}/lib/${MKL_TARGET_ARCH}:${LIBRARY_PATH}"
else
    setenv LIBRARY_PATH "${CPRO_PATH}/compiler/lib/${MKL_TARGET_ARCH}:${MKLROOT}/lib/${MKL_TARGET_ARCH}"
endif

if ( ${?NLSPATH} ) then
    setenv NLSPATH "${MKLROOT}/lib/${MKL_TARGET_ARCH}/locale/%l_%t/%N:${NLSPATH}"
else
    setenv NLSPATH "${MKLROOT}/lib/${MKL_TARGET_ARCH}/locale/%l_%t/%N"
endif

if ( ${?MANPATH} ) then
    setenv MANPATH "${CPRO_PATH}/man/en_US:${MANPATH}"
else
    setenv MANPATH "${CPRO_PATH}/man/en_US:`manpath`"
endif

if ( ${?INCLUDE} ) then
    setenv INCLUDE "${MKLROOT}/include:${INCLUDE}"
else
    setenv INCLUDE "${MKLROOT}/include"
endif

if ( ${?CPATH} ) then
    setenv CPATH "${MKLROOT}/include:${CPATH}"
else
    setenv CPATH "${MKLROOT}/include"
endif

if ( "${MKL_MOD}" == "${MOD_NAME}" )  then
    if ( "${MKL_TARGET_ARCH}" == "ia32" ) then
        setenv MKL_LP64_ILP64
    else
        if ( "$MKL_LP64_ILP64" == "" ) then
            setenv MKL_LP64_ILP64 lp64
        endif
    endif
    setenv MKL_MOD :${MKLROOT}/include/${MKL_TARGET_ARCH}/${MKL_LP64_ILP64}
endif
if ( ${?FPATH} ) then
    setenv FPATH "${MKLROOT}/include${MKL_MOD}:${FPATH}"
else
    setenv FPATH "${MKLROOT}/include${MKL_MOD}"
endif

if ( "${MKL_VERBOSE}" == "verbose" ) then
    echo LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
    echo LIBRARY_PATH=${LIBRARY_PATH}
    echo NLSPATH=${NLSPATH}
    echo MANPATH=${MANPATH}
    echo INCLUDE=${INCLUDE}
    echo FPATH=${FPATH}
    echo CPATH=${CPATH}
endif

goto End

Help:
    echo ""
    echo "Syntax:"
    echo "  $SCRIPT_NAME <arch> [MKL_interface] [${MOD_NAME}]"
    echo ""
    echo "   <arch> must be is one of the following"
    echo "       ia32         : Set up for IA-32 target"
    echo "       intel64      : Set up for Intel(R) 64 target"
    echo ""
    echo "   ${MOD_NAME} (optional) - set path to MKL F95 modules"
    echo ""
    echo "   MKL_interface (optional) - MKL programming interface for intel64"
    echo "                              Not applicable without ${MOD_NAME}"
    echo "       lp64         : 4 bytes integer (default)"
    echo "       ilp64        : 8 bytes integer"
    echo ""
    exit 1;

End:
