#! /bin/sh
#
# Copyright (C) 2003-2010 Intel Corporation. All rights reserved.
#
# The information and source code contained herein is the exclusive property
# of Intel Corporation and may not be disclosed, examined, or reproduced in
# whole or in part without explicit written authorization from the Company.
#

export MKLROOT=<INSTALLDIR>/mkl

CPRO_PATH=${MKLROOT/\/mkl/}

MOD_NAME=mod

SCRIPT_NAME=$0
MKL_LP64_ILP64=
MKL_MOD=
MKL_TARGET_ARCH=
MKL_VERBOSE=

mkl_help() {
    echo ""
    echo "Syntax:"
    echo "  $SCRIPT_NAME <arch> [MKL_interface] [${MOD_NAME}]"
    echo ""
    echo "   <arch> must be one of the following"
    echo "       ia32         : Set up for IA-32 target"
    echo "       intel64      : Set up for Intel(R) 64 target"
    echo ""
    echo "   ${MOD_NAME} (optional) - set path to MKL F95 modules"
    echo ""
    echo "   MKL_interface (optional) - MKL programming interface for intel64"
    echo "                              Not applicable without ${MOD_NAME}"
    echo "       lp64         : 4 bytes integer (default)"
    echo "       ilp64        : 8 bytes integer"
    echo ""
}

if [ "$1" == "" ]; then
    mkl_help
else

    BAD_SWITCH=""
    while [ $1 ]; do
       if   [ "$1" == "ia32" ]       ; then MKL_TARGET_ARCH=ia32;
       elif [ "$1" == "intel64" ]    ; then MKL_TARGET_ARCH=intel64;
       elif [ "$1" == "lp64" ]       ; then MKL_LP64_ILP64=lp64;
       elif [ "$1" == "ilp64" ]      ; then MKL_LP64_ILP64=ilp64;
       elif [ "$1" == "${MOD_NAME}" ]; then MKL_MOD=${MOD_NAME};
       elif [ "$1" == "verbose" ]    ; then MKL_VERBOSE=verbose;
       else
           BAD_SWITCH=$1
           break 10
       fi
       shift;
    done

    if [ "${BAD_SWITCH}" != "" ]; then

        echo
        echo "ERROR: Unknown switch '${BAD_SWITCH}'"
        mkl_help

    else

        if [ "${MKL_TARGET_ARCH}" == "" ]; then

            echo
            echo "ERROR: architecture is not defined. Accepted values: ia32, intel64"
            mkl_help

        else # main path

            if [ -n "${LD_LIBRARY_PATH}" ]; then OLD_LD_LIBRARY_PATH=":${LD_LIBRARY_PATH}"; fi
            export LD_LIBRARY_PATH="${CPRO_PATH}/compiler/lib/${MKL_TARGET_ARCH}:${MKLROOT}/lib/${MKL_TARGET_ARCH}${OLD_LD_LIBRARY_PATH}"

            if [ -n "${LIBRARY_PATH}" ]; then OLD_LIBRARY_PATH=":${LIBRARY_PATH}"; fi
            export LIBRARY_PATH="${CPRO_PATH}/compiler/lib/${MKL_TARGET_ARCH}:${MKLROOT}/lib/${MKL_TARGET_ARCH}${OLD_LIBRARY_PATH}"

            if [ -n "${NLSPATH}" ]; then OLD_NLSPATH=":${NLSPATH}"; fi
            export NLSPATH="${MKLROOT}/lib/${MKL_TARGET_ARCH}/locale/%l_%t/%N${OLD_NLSPATH}"

            if [ -z "${MANPATH}" ]; then
                export MANPATH="${CPRO_PATH}/man/en_US:`manpath`"
            else
                export MANPATH="${CPRO_PATH}/man/en_US:${MANPATH}"
            fi

            if [ -n "${INCLUDE}" ]; then OLD_INCLUDE=":${INCLUDE}"; fi
            export INCLUDE="${MKLROOT}/include${OLD_INCLUDE}"

            if [ -n "$CPATH" ]; then OLD_CPATH=":${CPATH}"; fi
            export CPATH="${MKLROOT}/include${OLD_CPATH}"

            if [ "${MKL_MOD}" == "${MOD_NAME}" ]; then
                if [ "${MKL_TARGET_ARCH}" == "ia32" ]; then
                    MKL_LP64_ILP64=
                else
                    if [ "$MKL_LP64_ILP64" == "" ]; then
                        MKL_LP64_ILP64=lp64
                    fi
                fi
                MKL_MOD=:${MKLROOT}/include/${MKL_TARGET_ARCH}/${MKL_LP64_ILP64}
            fi
            if [ -n "$FPATH" ]; then OLD_FPATH=":${FPATH}"; fi
            export FPATH="${MKLROOT}/include${MKL_MOD}${OLD_FPATH}"

            ########

            if [ "${MKL_VERBOSE}" == "verbose" ]; then
                echo LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
                echo LIBRARY_PATH=${LIBRARY_PATH}
                echo NLSPATH=${NLSPATH}
                echo MANPATH=${MANPATH}
                echo INCLUDE=${INCLUDE}
                echo FPATH=${FPATH}
                echo CPATH=${CPATH}
            fi
        fi
    fi
fi
