!*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       Example of using MKL DFTI in Fortran.
!
!       Single precision multiple split complex out-of-place 3D transform.
!
!*****************************************************************************

program single_split_complex_3d_ex2

  use mkl_dfti, forget => DFTI_SINGLE, DFTI_SINGLE => DFTI_SINGLE_R

  integer, parameter :: N(3) = (/5,10,20/) ! Sizes of 3D transform
  integer, parameter :: M = 4 ! Number of transforms

  type(dfti_descriptor), pointer :: descf,descb
  real*4, allocatable :: xr(:,:,:,:), xi(:,:,:,:)
  real*4, allocatable :: yr(:,:,:,:), yi(:,:,:,:)
  complex*8 :: got, exd
  integer n1, n2, n3, mm, xstride(4), ystride(4), xdist, ydist, status

  ! Allocate i/o arrays.
  allocate(xr(M,N(1),N(2),N(3)),xi(M,N(1),N(2),N(3)))
  allocate(yr(N(1),N(2),N(3),M),yi(N(1),N(2),N(3),M))

  ! Set up stride arrays needed for plan creation
  xdist      = 1
  xstride(1) = 0                ! Offset
  xstride(2) = size(xr,dim=1)
  xstride(3) = size(xr,dim=1)*size(xr,dim=2)
  xstride(4) = size(xr,dim=1)*size(xr,dim=2)*size(xr,dim=3)
  ystride(1) = 0                ! Offset
  ystride(2) = 1
  ystride(3) = size(yr,dim=1)
  ystride(4) = size(yr,dim=1)*size(yr,dim=2)
  ydist      = size(yr,dim=1)*size(yr,dim=2)*size(yr,dim=3)

  print *,"Single precision multiple split complex out-of-place 3D transform"
  print *,"Configuration parameters:"
  print *," DFTI_PRECISION            = DFTI_SINGLE"
  print *," DFTI_FORWARD_DOMAIN       = DFTI_COMPLEX"
  print *," DFTI_PLACEMENT            = DFTI_NOT_INPLACE"
  print *," DFTI_DIMENSION            = 3"
  print '("  DFTI_LENGTHS              = (/"I0","I0","I0"/)")', N
  print '("  DFTI_INPUT_STRIDES        = (/"I0,3(","I0)"/)")',xstride
  print '("  DFTI_OUTPUT_STRIDES       = (/"I0,3(","I0)"/)")',ystride
  print '("  DFTI_NUMBER_OF_TRANSFORMS = "I0)', M
  print '("  DFTI_INPUT_DISTANCE       = "I0)', xdist
  print '("  DFTI_OUTPUT_DISTANCE      = "I0)', ydist

  ! Create descriptor for forward transform.
  status = DftiCreateDescriptor(descf,DFTI_SINGLE,DFTI_COMPLEX,3,N)
  if (status /= 0) call nogo("DftiCreateDescriptor(descf)",status)

  status = DftiSetValue(descf,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(descf,COMPLEX_STORAGE)",status)

  status = DftiSetValue(descf,DFTI_PLACEMENT,DFTI_NOT_INPLACE)
  if (status /= 0) call nogo("DftiSetValue(descf,PLACEMENT)",status)

  status = DftiSetValue(descf,DFTI_INPUT_STRIDES,xstride)
  if (status /= 0) call nogo("DftiSetValue(descf,INPUT_STRIDES)",status)

  status = DftiSetValue(descf,DFTI_OUTPUT_STRIDES,ystride)
  if (status /= 0) call nogo("DftiSetValue(descf,OUTPUT_STRIDES)",status)

  status = DftiSetValue(descf,DFTI_NUMBER_OF_TRANSFORMS,M)
  if (status /= 0) call nogo("DftiSetValue(descf,NUMBER_OF_TRANSFORMS)",status)

  status = DftiSetValue(descf,DFTI_INPUT_DISTANCE,xdist)
  if (status /= 0) call nogo("DftiSetValue(descf,INPUT_DISTANCE)",status)

  status = DftiSetValue(descf,DFTI_OUTPUT_DISTANCE,ydist)
  if (status /= 0) call nogo("DftiSetValue(descf,OUTPUT_DISTANCE)",status)

  status = DftiCommitDescriptor(descf)
  if (status /= 0) call nogo("DftiCommitDescriptor(descf)",status)

  ! Create descriptor for backward transform.
  ! Note: input/output strides and distances are exchanged.
  status = DftiCreateDescriptor(descb,DFTI_SINGLE,DFTI_COMPLEX,3,N)
  if (status /= 0) call nogo("DftiCreateDescriptor(descb)",status)

  status = DftiSetValue(descb,DFTI_COMPLEX_STORAGE,DFTI_REAL_REAL)
  if (status /= 0) call nogo("DftiSetValue(descb,COMPLEX_STORAGE)",status)

  status = DftiSetValue(descb,DFTI_PLACEMENT,DFTI_NOT_INPLACE)
  if (status /= 0) call nogo("DftiSetValue(descb,PLACEMENT)",status)

  status = DftiSetValue(descb,DFTI_INPUT_STRIDES,ystride)
  if (status /= 0) call nogo("DftiSetValue(descb,INPUT_STRIDES)",status)

  status = DftiSetValue(descb,DFTI_OUTPUT_STRIDES,xstride)
  if (status /= 0) call nogo("DftiSetValue(descb,OUTPUT_STRIDES)",status)

  status = DftiSetValue(descb,DFTI_NUMBER_OF_TRANSFORMS,M)
  if (status /= 0) call nogo("DftiSetValue(descb,NUMBER_OF_TRANSFORMS)",status)

  status = DftiSetValue(descb,DFTI_INPUT_DISTANCE,ydist)
  if (status /= 0) call nogo("DftiSetValue(descb,INPUT_DISTANCE)",status)

  status = DftiSetValue(descb,DFTI_OUTPUT_DISTANCE,xdist)
  if (status /= 0) call nogo("DftiSetValue(descb,OUTPUT_DISTANCE)",status)

  status = DftiCommitDescriptor(descb)
  if (status /= 0) call nogo("DftiCommitDescriptor(descb)",status)

  ! Enter input data.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do n3 = 1, N(3)
           xr(:,n1,n2,n3) = sin(n1+n2+n3+0.1)
           xi(:,n1,n2,n3) = sin(n1+n2+n3-0.1)
        enddo
     enddo
  enddo

  ! Compute forward transform.
  status = DftiComputeForward(descf, &
       & xr(:,1,1,1),xi(:,1,1,1),yr(:,1,1,1),yi(:,1,1,1))
  if (status /= 0) call nogo("DftiComputeForward(descf)",status)

  ! Compute backward transform.
  status = DftiComputeBackward(descb, &
       & yr(:,1,1,1),yi(:,1,1,1),xr(:,1,1,1),xi(:,1,1,1))
  if (status /= 0) call nogo("DftiComputeForward",status)

  ! Discard the descriptors.
  status = DftiFreeDescriptor(descf)
  if (status /= 0) call nogo("DftiFreeDescriptor(descf)",status)
  status = DftiFreeDescriptor(descb)
  if (status /= 0) call nogo("DftiFreeDescriptor(descb)",status)

  ! Check the result: should be simply scaled input.
  do n1 = 1, N(1)
     do n2 = 1, N(2)
        do n3 = 1, N(3)
           do mm = 1, M
              got = cmplx(xr(mm,n1,n2,n3),xi(mm,n1,n2,n3))
              exd = cmplx(sin(n1+n2+n3+0.1),sin(n1+n2+n3-0.1)) &
                   & * N(1)*N(2)*N(3)
              if (.not.(areldiff(got,exd) < 1e-4)) then
                 call error(mm,n1,n2,n3,exd,got)
              endif
           enddo
        enddo
     enddo
  enddo

  ! Deallocate data arrays.
  deallocate( xr, xi )
  deallocate( yr, yi )
  print *, 'TEST PASSED'
  stop 0

contains

  ! A relative difference between two complex numbers
  real*4 function areldiff(a,b)
    complex*8 a,b
    real*4 norm
    norm = max(abs(a),abs(b))
    if (norm > 0) then
       areldiff = abs(a-b) / norm
    else
       areldiff = abs(a-b)
    endif
  end function areldiff

  ! Report error and terminate
  subroutine error(mm,n1,n2,n3,exd,got)
    integer :: mm,n1,n2,n3
    complex*8 :: exd, got

    print '("Result("I3";"I3","I3","I3"): "$)', mm,n1,n2,n3
    print '("expected ("G11.4","G11.4") "$)', exd
    print '("got ("G11.4","G11.4")")', got
    print *, 'TEST FAILED'
    stop 1
  end subroutine error

  ! Report execution error and terminate
  subroutine nogo(string,status)
    character*(*) string
    integer status
    print *,"Cannot continue: ",string," failed with status ", status
    stop 1
  end subroutine nogo

end program single_split_complex_3d_ex2
