!/*******************************************************************************
!                             INTEL CONFIDENTIAL
!  Copyright(C) 2003-2010 Intel Corporation. All Rights Reserved.
!  The source code contained  or  described herein and all documents related to
!  the source code ("Material") are owned by Intel Corporation or its suppliers
!  or licensors.  Title to the  Material remains with  Intel Corporation or its
!  suppliers and licensors. The Material contains trade secrets and proprietary
!  and  confidential  information of  Intel or its suppliers and licensors. The
!  Material  is  protected  by  worldwide  copyright  and trade secret laws and
!  treaty  provisions. No part of the Material may be used, copied, reproduced,
!  modified, published, uploaded, posted, transmitted, distributed or disclosed
!  in any way without Intel's prior express written permission.
!  No license  under any  patent, copyright, trade secret or other intellectual
!  property right is granted to or conferred upon you by disclosure or delivery
!  of the Materials,  either expressly, by implication, inducement, estoppel or
!  otherwise.  Any  license  under  such  intellectual property  rights must be
!  express and approved by Intel in writing.
!
!*******************************************************************************
!  Content:
!    Calculation of min/max estimators  Example Program Text
!******************************************************************************/

      include 'mkl_vsl.f90'
      include "errcheck.inc"
      include "generatedata.inc"

      program MKL_VSL_TEST

      USE MKL_VSL_TYPE
      USE MKL_VSL

      integer,parameter :: DIM = 3       ! Task dimension
      integer,parameter :: NN  = 1000    ! Number of observations

      TYPE(VSL_SS_TASK) task
      integer p
      integer n
      integer x_storage
      real(kind=8) x(DIM,NN)
      real(kind=8) min_est(DIM), max_est(DIM)
      real(kind=8) a, sigma
      integer i, j, errcode
      integer task_method
      integer(kind=8) task_params
      integer errnums

!     ***** Initialize parameters of Summary Statistics task *****
      p           = DIM
      n           = NN
      x_storage   = VSL_SS_MATRIX_STORAGE_COLS
      a           = 0.0
      sigma       = 1.0
      task_params = IOR( VSL_SS_MAX, VSL_SS_MIN )
      task_method = VSL_SS_METHOD_FAST
      errcode     = 0
      errnums     = 0

!     ***** Generate data set using VSL Gaussian RNG
!           with a = 0 and sigma = 1 *****
      errcode = dGenerateGaussianData( p, n, x, a, sigma )
      call CheckVslError( errcode )

!     ***** Set initial values of the estimates *****
      do i = 1, p
        min_est(i) = x(i, 1)
        max_est(i) = x(i, 1)
      end do

!     ***** Create Summary Statistics task *****
      errcode = vsldssnewtask( task, p, n, x_storage, x )
      call CheckVslError( errcode )

!     ***** Edit task parameters for min and max computation *****
      errcode = vsldssedittask( task, VSL_SS_ED_MIN, min_est )
      call CheckVslError( errcode )

      errcode = vsldssedittask( task, VSL_SS_ED_MAX, max_est )
      call CheckVslError( errcode )

!     ***** Compute min and max estimates using FAST method *****
      errcode = vsldsscompute( task, task_params , task_method )
      call CheckVslError( errcode )

!     ***** Comparison of observations with min and max estimates *****
      do i = 1, p
         do j = 1, n
            if ( x(i, j) < min_est(i) ) then
               errnums = errnums + 1
            end if
            if ( x(i, j) > max_est(i) ) then
               errnums = errnums + 1
            end if
         end do
      end do

!     ***** Printing results *****
      print 9, 'Task dimension :         ', p
      print 9, 'Number of observations : ', n
      print *, ''
      print *,'                 Min        Max'
      do i = 1, p
         print 10, 'Variable #', i ,':  ', min_est(i), max_est(i)
      end do

      print *, ''
      print *, ''

!     ***** Printing summary of the test *****
      if ( errnums == 0 ) then
         print *, ' All observations are',                                     &
     & ' within ranges for all dimensions'
      else
         print 11, ' Error: There are ', errnums,                              &
     & ' observations beyond the ranges'
         stop 1
      end if

!     ***** Delete Summary Statistics task *****
      errcode = vslssdeletetask( task )
      call CheckVslError( errcode )

      call MKL_FREE_BUFFERS()

9     format (A,I4)
10    format (A,I2,A,F10.6,F10.6)
11    format (A,I4,A)

      end
