!*******************************************************************************
!                              INTEL CONFIDENTIAL
!  Copyright(C) 2001-2010 Intel Corporation. All Rights Reserved.
!  The source code contained  or  described herein and all documents related to
!  the source code ("Material") are owned by Intel Corporation or its suppliers
!  or licensors.  Title to the  Material remains with  Intel Corporation or its
!  suppliers and licensors. The Material contains trade secrets and proprietary
!  and  confidential  information of  Intel or its suppliers and licensors. The
!  Material  is  protected  by  worldwide  copyright  and trade secret laws and
!  treaty  provisions. No part of the Material may be used, copied, reproduced,
!  modIFied, published, uploaded, posted, transmitted, distributed or disclosed
!  in any way without Intel's prior express written permission.
!  No license  under any  patent, copyright, trade secret or other intellectual
!  property right is granted to or conferred upon you by disclosure or delivery
!  of the Materials,  either expressly, by implication, inducement, estoppel or
!  otherwise.  Any  license  under  such  intellectual property  rights must be
!  express and approved by Intel in writing.
!
!*******************************************************************************
! Content:
! Example of 1-dimension linear correlation operation on singe precision data
!*******************************************************************************/

      INCLUDE 'mkl_vsl.fi'

      PROGRAM vslscorrelation_1d_auto
      USE MKL_VSL_TYPE
      USE MKL_VSL
      IMPLICIT NONE
C
      INTEGER status,i
C
      TYPE(VSL_CORR_TASK) task
      INTEGER mode,xshape,yshape,zshape
      REAL*4 x(4),y(8),z(11),e(11)
      DATA x/1,2,3,4/
      DATA y/11,12,13,14,15,16,17,18/
      DATA z/0,0,0,0,0,0,0,0,0,0,0/
      DATA e/44,81,110,130,140,150,160,170,104,53,18/
      DATA xshape/4/, yshape/8/, zshape/11/

      mode = VSL_CORR_MODE_AUTO

C
C  Create task descriptor (create descriptor of problem)
C
      status = vslscorrnewtask1d(task,mode,xshape,yshape,zshape)
      IF (status .NE. VSL_STATUS_OK) THEN
          PRINT *,"ERROR: creation of job failed, exit with ", status
          STOP 1
      ENDIF
C
C  Execute task (Calculate one dimension correlation of two arrays) 
C
      status = vslscorrexec1d(task,x,1,y,1,z,1)
      IF (status .NE. VSL_STATUS_OK) THEN
          PRINT *,"ERROR: job status bad, exit with", status
          STOP 1
      ENDIF
C
C  Delete task object (delete descriptor of problem)
C
      status = vslcorrdeletetask(task)
      IF (status .NE. VSL_STATUS_OK) THEN
          PRINT *,"ERROR: failed to delete task, exit with", status
          STOP 1
      ENDIF
C
C  Check results for correctness:
C
      DO i=1,zshape
         IF (abs(z(i)-e(i)) .GT. e(i)*1e-5) THEN
            PRINT *, "ERROR:"
            PRINT *, "      i :", i
            PRINT *, "    z(i):", z(i)
            PRINT *, "expected:", e(i)
            PRINT *, "EXAMPLE FAILED"
            STOP 1
         END IF
      END DO
C
      PRINT *, "EXAMPLE PASSED"
      STOP 0
      END
