/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2006-2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * fftwnd_create_plan_specific - FFTW2 wrapper to MKL.
 *
 ******************************************************************************
 */

#include "fftw2_mkl.h"

fftwnd_plan
fftwnd_create_plan_specific(int rank, const int *n, fftw_direction dir,
                            int flags, fftw_complex *in, int istride,
                            fftw_complex *out, int ostride)
{
    MKL_LONG err;
    fftw_plan_mkl *plan = NULL;
    int i;

    UNUSED(in);
    UNUSED(out);

    plan = (fftw_plan_mkl *)fftw_malloc(sizeof(fftw_plan_mkl));
    if (plan == NULL)
        return NULL;

    plan->howmany = 1;
    plan->idist = 0;
    plan->inplace = flags & FFTW_IN_PLACE;
    plan->istride = istride;
    plan->mkl_desc = NULL;
    plan->n = NULL;
    plan->nthreads = MKL_Domain_Get_Max_Threads(MKL_FFT);
    plan->odist = 0;
    plan->ostride = ostride;
    plan->rank = rank;
    plan->readonly = flags & FFTW_THREADSAFE;
    plan->sign = dir;

    if (rank > DFTI_MAX_RANK)
    {
        err = DFTI_UNIMPLEMENTED;
        goto cannot_commit;
    }
    
    plan->n = (MKL_LONG *)fftw_malloc(rank * sizeof(MKL_LONG));
    if (plan->n == NULL)
        goto cannot_commit;

    for (i = 0; i < rank; i++)
    {
        plan->n[i] = (MKL_LONG)n[i];
    }

    if (rank == 1)
    {
        err = DftiCreateDescriptor(&(plan->mkl_desc), DFTI_PRECISION,
                 DFTI_COMPLEX, rank, plan->n[0]);
    }
    else
    {
        err = DftiCreateDescriptor(&(plan->mkl_desc), DFTI_PRECISION,
                 DFTI_COMPLEX, rank, plan->n);
    }
    if (err) goto cannot_commit;

    if (!(plan->inplace))
    {
        err = DftiSetValue(plan->mkl_desc, DFTI_PLACEMENT, DFTI_NOT_INPLACE);
        if (err) goto cannot_commit;
    }
    
    /* Set input strides per row-major format */
    {
        MKL_LONG strides[1+DFTI_MAX_RANK];
        
        strides[0] = 0;
        strides[rank] = istride;
        for (i = rank - 1; i > 0; i--)
        {
            strides[i] = plan->n[i] * strides[i + 1];
        }

        err = DftiSetValue(plan->mkl_desc, DFTI_INPUT_STRIDES, strides);
        if (err) goto cannot_commit;
    }

    /* Set output strides per row-major format */
    if (!(plan->inplace))
    {
        MKL_LONG strides[1+DFTI_MAX_RANK];
        
        strides[0] = 0;
        strides[rank] = ostride;
        for (i = rank - 1; i > 0; i--)
        {
            strides[i] = plan->n[i] * strides[i + 1];
        }

        err = DftiSetValue(plan->mkl_desc, DFTI_OUTPUT_STRIDES, strides);
        if (err) goto cannot_commit;
    }


    err = DftiCommitDescriptor(plan->mkl_desc);
    if (err) goto cannot_commit;

    return (fftwnd_plan)plan;

cannot_commit:
    fftwnd_destroy_plan( (fftwnd_plan)plan );
    return NULL;
}
