/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * MPI FFTW3 wrappers to MKL.
 *
 ******************************************************************************
 */

#include "fftw3-mpi_mkl.h"

static void register_plan(fftw_mkl_plan p)
{
    registered_plan *rp = fftw_malloc(sizeof(registered_plan));
    if (NULL == rp) return;
    rp->plan = p;
    rp->prev = NULL;
    if (NULL != (rp->next = fftw3_mpi_mkl.registered_plans))
        rp->next->prev = rp;
    fftw3_mpi_mkl.registered_plans = rp;
}

static int unregister_plan(fftw_mkl_plan p)
{
    registered_plan *rp = fftw3_mpi_mkl.registered_plans;

    while ((NULL != rp) && (rp->plan != p))
        rp = rp->next;
    if (NULL == rp)
        return 0;

    if (NULL != rp->prev)
        rp->prev->next = rp->next;
    else
        fftw3_mpi_mkl.registered_plans = rp->next;

    if (NULL != rp->next)
        rp->next->prev = rp->prev;

    fftw_free(rp);

    return 1;
}

static void
fftw_mpi_delete_plan(fftw_mkl_plan p)
{
    if (0 != unregister_plan(p))
    {
        MKL_LONG status;
        DFTI_DESCRIPTOR_DM_HANDLE desc = (DFTI_DESCRIPTOR_DM_HANDLE)p->desc;
        status = DftiFreeDescriptorDM(&desc);
        /* FFTW does not have means to report a failure when deleting a plan */
        fftw_free(p);
        return;
    }
    fftw3_mpi_mkl.fftw_delete_plan(p);
}

FFTW_EXTERN void FFTW_MPI_MANGLE(init)(void)
{
    /* Fix up the delete_plan function */
    fftw3_mpi_mkl.fftw_delete_plan = fftw3_mkl.delete_plan;
    fftw3_mkl.delete_plan = fftw_mpi_delete_plan;
}

FFTW_EXTERN void FFTW_MPI_MANGLE(cleanup)(void)
{
    /* Restore the delete_plan function */
    fftw3_mkl.delete_plan = fftw3_mpi_mkl.fftw_delete_plan;
}

fftw3_mpi_mkl_s fftw3_mpi_mkl = {
    NULL, /* registered_plan *registered_plans */
    register_plan, /* void (*register_plan)(fftw_mkl_plan p) */
    unregister_plan, /* int (*unregister_plan)(fftw_mkl_plan p) */
    NULL /* void (*fftw_delete_plan)(fftw_mkl_plan p) */
};

