/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * MPI FFTW3 wrappers to MKL.
 *
 ******************************************************************************
 */

#include "fftw3-mpi_mkl.h"

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_many_transposed)
     (int rnk, const ptrdiff_t *n, ptrdiff_t howmany,
      ptrdiff_t block0, ptrdiff_t block1, MPI_Comm comm,
      ptrdiff_t *local_n0, ptrdiff_t *local_0_start,
      ptrdiff_t *local_n1, ptrdiff_t *local_1_start)
{
    return 0; /* not implemented */
}


FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_many)
     (int rnk, const ptrdiff_t *n, ptrdiff_t howmany,
      ptrdiff_t block0, MPI_Comm comm,
      ptrdiff_t *local_n0, ptrdiff_t *local_0_start)
{
    return 0; /* not implemented */
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_transposed)
     (int rnk, const ptrdiff_t *n, MPI_Comm comm,
      ptrdiff_t *local_n0, ptrdiff_t *local_0_start,
      ptrdiff_t *local_n1, ptrdiff_t *local_1_start)
{
    return 0; /* not implemented */
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size)
     (int rnk, const ptrdiff_t *n, MPI_Comm comm,
      ptrdiff_t *local_n0, ptrdiff_t *local_0_start)
{
    MKL_LONG *dims = NULL;
    DFTI_DESCRIPTOR_DM_HANDLE desc = NULL;
    MKL_LONG status, value;
    ptrdiff_t ret = 0;
    int i;

    dims = fftw_malloc(sizeof(MKL_LONG)*rnk);
    if (NULL == dims) goto cleanup;

    for (i=0; i<rnk; i++)
        dims[i] = n[i];

    status = DftiCreateDescriptorDM(MKL_comm_c2m(comm), &desc, MKL_PREC, DFTI_COMPLEX, (MKL_LONG)rnk, dims);
    if (DFTI_NO_ERROR != status) return ret;
    status = DftiSetValueDM(desc, DFTI_PLACEMENT, DFTI_NOT_INPLACE);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiCommitDescriptorDM(desc);
    if (DFTI_NO_ERROR != status) goto cleanup;

    status = DftiGetValueDM(desc, CDFT_LOCAL_NX, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    *local_n0 = value;

    status = DftiGetValueDM(desc, CDFT_LOCAL_X_START, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    *local_0_start = value;

    status = DftiGetValueDM(desc, CDFT_LOCAL_SIZE, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    ret = value;

cleanup:
    if (NULL != dims) fftw_free(dims);
    DftiFreeDescriptorDM(&desc);
    return ret;
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_many_1d)(
     ptrdiff_t n0, ptrdiff_t howmany,
     MPI_Comm comm, int sign, unsigned flags,
     ptrdiff_t *local_ni, ptrdiff_t *local_i_start,
     ptrdiff_t *local_no, ptrdiff_t *local_o_start)
{
    return 0; /* not implemented */
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_1d)(
     ptrdiff_t n0, MPI_Comm comm, int sign, unsigned flags,
     ptrdiff_t *local_ni, ptrdiff_t *local_i_start,
     ptrdiff_t *local_no, ptrdiff_t *local_o_start)
{
    DFTI_DESCRIPTOR_DM_HANDLE desc;
    MKL_LONG status, value;
    ptrdiff_t ret = 0;

    status = DftiCreateDescriptorDM(MKL_comm_c2m(comm), &desc, MKL_PREC, DFTI_COMPLEX, (MKL_LONG)1, (MKL_LONG)n0);
    if (DFTI_NO_ERROR != status) return ret;
    status = DftiSetValueDM(desc, DFTI_PLACEMENT, DFTI_NOT_INPLACE);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiCommitDescriptorDM(desc);
    if (DFTI_NO_ERROR != status) goto cleanup;

    status = DftiGetValueDM(desc, CDFT_LOCAL_NX, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    *local_ni = value;

    status = DftiGetValueDM(desc, CDFT_LOCAL_X_START, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    *local_i_start = value;

    status = DftiGetValueDM(desc, CDFT_LOCAL_OUT_NX, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    *local_no = value;

    status = DftiGetValueDM(desc, CDFT_LOCAL_OUT_X_START, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    *local_o_start = value;

    status = DftiGetValueDM(desc, CDFT_LOCAL_SIZE, &value);
    if (DFTI_NO_ERROR != status) goto cleanup;
    ret = value;

cleanup:
    DftiFreeDescriptorDM(&desc);
    return ret;
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_2d)(
     ptrdiff_t n0, ptrdiff_t n1, MPI_Comm comm,
     ptrdiff_t *local_n0, ptrdiff_t *local_0_start)
{
    const ptrdiff_t n[] = {n0, n1};
    return FFTW_MPI_MANGLE(local_size)(2, n, comm, local_n0, local_0_start);
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_2d_transposed)(
     ptrdiff_t n0, ptrdiff_t n1, MPI_Comm comm,
     ptrdiff_t *local_n0, ptrdiff_t *local_0_start,
     ptrdiff_t *local_n1, ptrdiff_t *local_1_start)
{
    const ptrdiff_t n[] = {n0, n1};
    return FFTW_MPI_MANGLE(local_size_transposed)(2, n, comm, local_n0, local_0_start, local_n1, local_1_start);
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_3d)(
     ptrdiff_t n0, ptrdiff_t n1, ptrdiff_t n2, MPI_Comm comm,
     ptrdiff_t *local_n0, ptrdiff_t *local_0_start)
{
    const ptrdiff_t n[] = {n0, n1, n2};
    return FFTW_MPI_MANGLE(local_size)(3, n, comm, local_n0, local_0_start);
}

FFTW_EXTERN ptrdiff_t FFTW_MPI_MANGLE(local_size_3d_transposed)(
     ptrdiff_t n0, ptrdiff_t n1, ptrdiff_t n2, MPI_Comm comm,
     ptrdiff_t *local_n0, ptrdiff_t *local_0_start,
     ptrdiff_t *local_n1, ptrdiff_t *local_1_start)
{
    const ptrdiff_t n[] = {n0, n1, n2};
    return FFTW_MPI_MANGLE(local_size_transposed)(3, n, comm, local_n0, local_0_start, local_n1, local_1_start);
}
