/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * MPI FFTW3 wrappers to MKL.
 *
 ******************************************************************************
 */

#include "fftw3-mpi_mkl.h"

static void execute_fwd(fftw_mkl_plan p)
{
    fftw_mkl_plan mkl_p = (fftw_mkl_plan)p;
    MKL_LONG status;

    status = DftiComputeForwardDM((DFTI_DESCRIPTOR_DM_HANDLE)mkl_p->desc, mkl_p->io[0], mkl_p->io[1]);

    /* FFTW does not have means to report a failure during execution */
}

static void execute_bwd(fftw_mkl_plan p)
{
    fftw_mkl_plan mkl_p = (fftw_mkl_plan)p;
    MKL_LONG status;

    status = DftiComputeBackwardDM((DFTI_DESCRIPTOR_DM_HANDLE)mkl_p->desc, mkl_p->io[0], mkl_p->io[1]);

    /* FFTW does not have means to report a failure during execution */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_many_transpose)
     (ptrdiff_t n0, ptrdiff_t n1,
      ptrdiff_t howmany, ptrdiff_t block0, ptrdiff_t block1,
      REAL_T *in, REAL_T *out, MPI_Comm comm, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_transpose)
     (ptrdiff_t n0, ptrdiff_t n1,
      REAL_T *in, REAL_T *out, MPI_Comm comm, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_many_dft)
     (int rnk, const ptrdiff_t *n, ptrdiff_t howmany,
      ptrdiff_t block, ptrdiff_t tblock, COMPLEX_T *in, COMPLEX_T *out,
      MPI_Comm comm, int sign, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft)
     (int rnk, const ptrdiff_t *n, COMPLEX_T *in, COMPLEX_T *out,
      MPI_Comm comm, int sign, unsigned flags)
{
    MKL_LONG *dims = NULL;
    DFTI_DESCRIPTOR_DM_HANDLE desc = NULL;
    MKL_LONG status;
    fftw_mkl_plan p = NULL;
    int i;

    dims = fftw_malloc(sizeof(MKL_LONG)*rnk);
    if (NULL == dims) goto cleanup;

    for (i=0; i<rnk; i++)
        dims[i] = n[i];

    status = DftiCreateDescriptorDM(MKL_comm_c2m(comm), &desc, MKL_PREC, DFTI_COMPLEX, (MKL_LONG)rnk, (rnk>1)?dims:(MKL_LONG*)*n);
    if (DFTI_NO_ERROR != status) return (FFTW_MANGLE(plan))p;
    status = DftiSetValueDM(desc, DFTI_PLACEMENT, (in != out) ? DFTI_NOT_INPLACE : DFTI_INPLACE);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiCommitDescriptorDM(desc);
    if (DFTI_NO_ERROR != status) goto cleanup;

    p = (fftw_mkl_plan)fftw_malloc(sizeof(*p));
    if (NULL == p) goto cleanup;

    p->desc = (DFTI_DESCRIPTOR_HANDLE)desc;
    p->execute = (FFTW_FORWARD == sign) ? execute_fwd : execute_bwd;
    p->io[0] = in;
    p->io[1] = out;

    fftw3_mpi_mkl.register_plan(p);
    return (FFTW_MANGLE(plan))p;

cleanup:
    if (NULL != dims) fftw_free(dims);
    DftiFreeDescriptorDM(&desc);
    return (FFTW_MANGLE(plan))p;
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_1d)
     (ptrdiff_t n0, COMPLEX_T *in, COMPLEX_T *out,
      MPI_Comm comm, int sign, unsigned flags)
{
    const ptrdiff_t n[] = {n0};
    return FFTW_MPI_MANGLE(plan_dft)(1, n, in, out, comm, sign, flags);
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_2d)
     (ptrdiff_t n0, ptrdiff_t n1, COMPLEX_T *in, COMPLEX_T *out,
      MPI_Comm comm, int sign, unsigned flags)
{
    const ptrdiff_t n[] = {n0, n1};
    return FFTW_MPI_MANGLE(plan_dft)(2, n, in, out, comm, sign, flags);
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_3d)
     (ptrdiff_t n0, ptrdiff_t n1, ptrdiff_t n2, COMPLEX_T *in, COMPLEX_T *out,
      MPI_Comm comm, int sign, unsigned flags)
{
    const ptrdiff_t n[] = {n0, n1, n2};
    return FFTW_MPI_MANGLE(plan_dft)(3, n, in, out, comm, sign, flags);
}


FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_many_r2r)
     (int rnk, const ptrdiff_t *n, ptrdiff_t howmany,
      ptrdiff_t iblock, ptrdiff_t oblock, REAL_T *in, REAL_T *out,
      MPI_Comm comm, const FFTW_MANGLE(r2r_kind) *kind, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_r2r)
     (int rnk, const ptrdiff_t *n, REAL_T *in, REAL_T *out,
      MPI_Comm comm, const FFTW_MANGLE(r2r_kind) *kind, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_r2r_2d)
     (ptrdiff_t n0, ptrdiff_t n1, REAL_T *in, REAL_T *out, MPI_Comm comm,
      FFTW_MANGLE(r2r_kind) kind0, FFTW_MANGLE(r2r_kind) kind1, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_r2r_3d)
     (ptrdiff_t n0, ptrdiff_t n1, ptrdiff_t n2,
      REAL_T *in, REAL_T *out, MPI_Comm comm, FFTW_MANGLE(r2r_kind) kind0,
      FFTW_MANGLE(r2r_kind) kind1, FFTW_MANGLE(r2r_kind) kind2, unsigned flags)
{
    return 0; /* not implemented */
}


FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_many_dft_r2c)
     (int rnk, const ptrdiff_t *n, ptrdiff_t howmany,
      ptrdiff_t iblock, ptrdiff_t oblock, REAL_T *in, COMPLEX_T *out,
      MPI_Comm comm, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_r2c)
     (int rnk, const ptrdiff_t *n, REAL_T *in, COMPLEX_T *out,
      MPI_Comm comm, unsigned flags)
{
    MKL_LONG *dims = NULL;
    DFTI_DESCRIPTOR_DM_HANDLE desc = NULL;
    MKL_LONG status;
    fftw_mkl_plan p = NULL;
    int i;

    dims = fftw_malloc(sizeof(MKL_LONG)*rnk);
    if (NULL == dims) goto cleanup;

    for (i=0; i<rnk; i++)
        dims[i] = n[i];

    status = DftiCreateDescriptorDM(MKL_comm_c2m(comm), &desc, MKL_PREC, DFTI_REAL, (MKL_LONG)rnk, (rnk>1)?dims:(MKL_LONG*)*n);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiSetValueDM(desc, DFTI_PLACEMENT, DFTI_NOT_INPLACE);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiCommitDescriptorDM(desc);
    if (DFTI_NO_ERROR != status) goto cleanup;

    p = (fftw_mkl_plan)fftw_malloc(sizeof(*p));
    if (NULL == p) goto cleanup;

    p->desc = (DFTI_DESCRIPTOR_HANDLE)desc;
    p->execute = execute_fwd;
    p->io[0] = in;
    p->io[1] = out;

    fftw3_mpi_mkl.register_plan(p);
    return (FFTW_MANGLE(plan))p;

cleanup:
    if (NULL != dims) fftw_free(dims);
    return (FFTW_MANGLE(plan))p;
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_r2c_2d)
     (ptrdiff_t n0, ptrdiff_t n1, REAL_T *in, COMPLEX_T *out,
      MPI_Comm comm, unsigned flags)
{
    const ptrdiff_t n[] = {n0, n1};
    return FFTW_MPI_MANGLE(plan_dft_r2c)(2, n, in, out, comm, flags);
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_r2c_3d)
     (ptrdiff_t n0, ptrdiff_t n1, ptrdiff_t n2, REAL_T *in, COMPLEX_T *out,
      MPI_Comm comm, unsigned flags)
{
    const ptrdiff_t n[] = {n0, n1, n2};
    return FFTW_MPI_MANGLE(plan_dft_r2c)(3, n, in, out, comm, flags);
}


FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_many_dft_c2r)
     (int rnk, const ptrdiff_t *n, ptrdiff_t howmany,
      ptrdiff_t iblock, ptrdiff_t oblock, COMPLEX_T *in, REAL_T *out,
      MPI_Comm comm, unsigned flags)
{
    return 0; /* not implemented */
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_c2r)
     (int rnk, const ptrdiff_t *n, COMPLEX_T *in, REAL_T *out,
      MPI_Comm comm, unsigned flags)
{
    MKL_LONG *dims = NULL;
    DFTI_DESCRIPTOR_DM_HANDLE desc = NULL;
    MKL_LONG status;
    fftw_mkl_plan p = NULL;
    int i;

    dims = fftw_malloc(sizeof(MKL_LONG)*rnk);
    if (NULL == dims) goto cleanup;

    for (i=0; i<rnk; i++)
        dims[i] = n[i];

    status = DftiCreateDescriptorDM(MKL_comm_c2m(comm), &desc, MKL_PREC, DFTI_REAL, (MKL_LONG)rnk, (rnk>1)?dims:(MKL_LONG*)*n);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiSetValueDM(desc, DFTI_PLACEMENT, DFTI_NOT_INPLACE);
    if (DFTI_NO_ERROR != status) goto cleanup;
    status = DftiCommitDescriptorDM(desc);
    if (DFTI_NO_ERROR != status) goto cleanup;

    p = (fftw_mkl_plan)fftw_malloc(sizeof(*p));
    if (NULL == p) goto cleanup;

    p->desc = (DFTI_DESCRIPTOR_HANDLE)desc;
    p->execute = execute_bwd;
    p->io[0] = in;
    p->io[1] = out;

    fftw3_mpi_mkl.register_plan(p);
    return (FFTW_MANGLE(plan))p;

cleanup:
    if (NULL != dims) fftw_free(dims);
    return (FFTW_MANGLE(plan))p;
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_c2r_2d)
     (ptrdiff_t n0, ptrdiff_t n1, COMPLEX_T *in, REAL_T *out,
      MPI_Comm comm, unsigned flags)
{
    const ptrdiff_t n[] = {n0, n1};
    return FFTW_MPI_MANGLE(plan_dft_c2r)(2, n, in, out, comm, flags);
}

FFTW_EXTERN FFTW_MANGLE(plan) FFTW_MPI_MANGLE(plan_dft_c2r_3d)
     (ptrdiff_t n0, ptrdiff_t n1, ptrdiff_t n2, COMPLEX_T *in, REAL_T *out,
      MPI_Comm comm, unsigned flags)
{
    const ptrdiff_t n[] = {n0, n1, n2};
    return FFTW_MPI_MANGLE(plan_dft_c2r)(3, n, in, out, comm, flags);
}
